import java.util.concurrent.Semaphore;

/**
 * This BasicPlayer class implements the basic functions of a player in the
 * algorithm guessing game. It implements the basic decision making, but
 * leaves the specific strategy to the subclasses.
 */
public abstract class BasicPlayer extends Thread {
  /**
   * This Semaphore is the object that hands out tickets that indicate that
   * this Player can participate. Each Player must have a ticket before
   * participating.
   */
  private Semaphore ticketSeller;
  /**
   * This GameBoard is where each Player can find the latest guess.
   */
  private GameBoard theBoard;

  /**
   * Remember the Semaphore that we need to use to get a permit to join the
   * game, and also the game board object where the dealer's guesses are
   * displayed.
   * @param tickets the Semaphore that hands out seats at the table (tickets)
   * @param board the GameBoard that holds information about the current state
   *          of the game (namely, the most recent guess)
   */
  public BasicPlayer(Semaphore tickets, GameBoard board) {
    ticketSeller = tickets;
    theBoard = board;
  }

  /**
   * Override the run() method to provide the overall decision making
   * capability.
   */
  public void run() {
    try {
      ticketSeller.acquire();
      System.out.println("Player " + getName() + " is at the table.");
      
      // stay at the table until we get a number we like

      boolean looking = true;
      synchronized (theBoard) {
        while (looking) {
          theBoard.wait();
          int value = theBoard.getGuess();
          if (isAcceptable(value)) {
            System.out.println("Player " + getName() + " finds that "
                + value + " is acceptable.");
            looking = false;
          } else {
            System.out.println("Player " + getName() + " finds that "
                + value + " is not acceptable.");
          }
        }
      }

      ticketSeller.release();
      System.out.println("Player " + getName() + " has left the table.");
    } catch (InterruptedException e) {
      System.out.println("Player " + getName()
          + " was interrupted and won't be joining the game.");
    }
  }

  /**
   * Evaluate the guessed number and decide if it's okay according to our
   * algorithm.
   * @param n the number to evaluate
   * @return true if okay, false if not
   */
  public abstract boolean isAcceptable(int n);
}
